/*
	DMXconverter - A channel converter widget for DMX512.
	Copyright © 2004-2007,2015 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA	 02110-1301	 USA
	
	DMXconverter - browser version 1.0
	16 May, 2015
	Copyright © 2004-2007,2015 Harry Whitfield
	mailto:g6auc@arrl.net
*/

/*global bkgd, sliderBkgd, dmxBinary, redButton, moveObj, newInput, newText,
	newImage
*/

/*properties
    hOffset, i, indexOf, length, onkeyup, onmouseup, substring, title, toString,
    vOffset, value
*/

var Slider = [];
var SliderButton = [];
var SliderLabel = [];

var sliderOn  = 408;
var sliderOff = 428;
var sliderSum = 836;
var sliderDif =	 20;

var Port = [];
var PortLabel = [];

var PortName = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H'];

var PortBase = [0, 512, 1024, 1536, 2048, 2560, 3072, 3584, 4096];

var range = [
	[[1, 512]], 
	[[1, 1], [5,  10], [51,  102], [513,  1024]],
	[[1, 1], [10, 15], [102, 153], [1025, 1536]],
	[[1, 2], [15, 20], [153, 204], [1537, 2048]],
	[[2, 2], [20, 25], [204, 256], [2049, 2560]],
	[[2, 3], [25, 30], [256, 307], [2561, 3072]],
	[[3, 3], [30, 35], [307, 358], [3073, 3584]],
	[[3, 4], [35, 40], [358, 409], [3585, 4096]]
];

function inRange(k, value) {
	var len = range[k].length, i;
	
	for (i = 0; i < len; i += 1) {
		if ((range[k][i][0] <= value) && (value <= range[k][i][1])) { return true; }
	}
	return false;
}

function parseInteger(s) {	// converts string of digits to number or NaN
	var n = s.length,
		i;
		
	if ((n === 0) || (n > 4)) { return NaN; }
	for (i = 0; i < n; i += 1) { if ('0123456789'.indexOf(s[i]) < 0) { return NaN; } }
	return Number(s);
}

function setSliders(dmx) {	// dmx 1..512	512 -> 0
	var i,
		hOffset,
		vOffset;
	
	dmx = dmx % 512;
	for (i = 0; i < 9; i += 1) {
		hOffset = SliderButton[i].hOffset;
		vOffset = sliderOff - sliderDif * (dmx % 2);
		moveObj(SliderButton[i], hOffset, vOffset);
		SliderButton[i].vOffset = vOffset;
		dmx = dmx >>> 1;
	}
}

function clearAllBut(k) { // clear all ports except port k
	var i;
	
	for (i = 0; i < 8; i += 1) { if (i !== k) { Port[i].value = ""; } }
	setSliders(0);
}

function computeData() {
	var i;
	
	for (i = 0; i < 7; i += 1) { Port[i + 1].value = (Number(Port[i].value) + 512).toString(10); }
}

function keyUp() {
	var i = this.i,
		num;
	
	function backspace() { Port[i].value = Port[i].value.substring(0, Port[i].value.length - 1); }
	
	if (Port[i].value !== "") {
		num = parseInteger(Port[i].value);
		
		if (isNaN(num)) {
			backspace();
		} else {
			if (!inRange(i, num)) {
				backspace();
			} else if (num <= PortBase[i]) {
				clearAllBut(i);
			} else {
				Port[0].value = (num - PortBase[i]).toString(10);
				computeData();
				setSliders(Number(Port[0].value));
			}
		}
	} else { clearAllBut(i); }
}

// newInput(hOffset, vOffset, width, height, value, zOrder, style)
// newText(hOffset, vOffset, width, height, value, zOrder, style)

function makePorts() {
	var i, hOffset, vOffset, style1, style2, value;

	style1 = "font-family:'Courier New';font-size:28px;font-weight:bold;background-color:#EAEAEA;color:#D60008;";
	style2 = "font-family:Helvetica;font-size:14px;font-weight:bold;background-color:transparent;border:none;color:#000000;";
	
	for (i = 0; i < 8; i += 1) {
		if (i < 4) { hOffset = 30;	} else { hOffset = 180; }
		vOffset = 80 * (i % 4) + 70;
		Port[i] = newInput(hOffset, vOffset, 26, 30, "", 10, style1);
		Port[i].i = i;
		Port[i].onkeyup = keyUp;

		if (i < 4) { hOffset = 22;	 } else { hOffset = 172; }
		vOffset = 80 * (i % 4) + 50;
		value = PortName[i] + ':' + (PortBase[i] + 1) + '-' + PortBase[i + 1];
		PortLabel[i] = newText(hOffset, vOffset, 65, 18, value, 10, style2);
		PortLabel[i].title = 'Channels ' + (PortBase[i] + 1) + '-' + PortBase[i + 1] + '.';
	}
}

function getSliders() {
	var dmx = 0,
		i;
		
	for (i = 8; i >= 0; i -= 1) {
		dmx = 2 * dmx;
		if (SliderButton[i].vOffset === sliderOn) { dmx += 1; } 
	}
	if (dmx === 0) { dmx = 512; }
	return dmx;
}

function setData() {
	Port[0].value = getSliders().toString(10);
	computeData();
}

function setSliderButton() {
	var vOffset = this.vOffset,
		hOffset = this.hOffset,
		newVOff = sliderSum - vOffset;
	
	moveObj(this, hOffset, newVOff);
	this.vOffset = newVOff;
	setData();
}

// newImage(hOffset, vOffset, width, height, src, zOrder, opacity)
// newText(hOffset, vOffset, width, height, value, zOrder, style)

function makeSliders() {
	var i,
		table = [" 1", " 2", " 4", " 8", "16", "32", "64", "128", "256"],
		hOffset,
		style = "font-family:Helvetica;font-size:12px;font-weight:bold;background-color:transparent;border:none;color:#000000;";
	
	for (i = 0; i < 9; i += 1) {
		hOffset = 28 * i + 37;
		Slider[i] = newImage(hOffset, 404, 3, 44, "Resources/slider.png", 20, 1.0);

		hOffset = 28 * i + 30;
		SliderButton[i] = newImage(hOffset, sliderOff, 17, 17, "Resources/sliderButton.png", 25, 1.0);
		SliderButton[i].hOffset = hOffset;
		SliderButton[i].vOffset = sliderOff;
		SliderButton[i].onmouseup = setSliderButton;

		hOffset = 28 * i + 29; 
		SliderLabel[i] = newText(hOffset, 375, 16, 18, table[i], 20, style);
	}
}

makeSliders();
makePorts();

setSliders(27);
setData();
